@@ -59,6 +59,19 @@ int *tclDummyMathPtr = (int *) matherr;
  */
 Widget *tclDummyWidgetPtr = (Widget*)XtAppInitialize;
 
+static Tcl_Interp *moatinterp = NULL;
+
+static void cleanup()
+{
+    if(NULL!=moatinterp) {
+      Tcl_Interp *p = moatinterp;
+      moatinterp = NULL;
+      Tcl_DeleteCommand(p,"send");
+      Tcl_DeleteInterp(p);
+    }
+}
+ 
+
 
 
 
@@ -105,6 +118,11 @@ Tcl_AppInit(interp)
 	return TCL_ERROR;
     }
 
+     moatinterp = interp;
+#ifdef __STDC__
+     atexit(cleanup);
+#endif
+
     /*
      * Call Tcl_CreateCommand for application-specific commands, if
      * they weren't already created by the init procedures called above.
