@@ -35,10 +35,15 @@ static char rcsid[] = "$Header";
 #ifndef MOTIF11
 #include <Xm/DragDrop.h>
 #endif
+
+#include <Xm/Xm.h>
+#include <Xm/AtomMgr.h>
 #include <Xm/Command.h>
 #include <Xm/List.h>
 #include <Xm/Text.h>
 #include <Xm/RowColumn.h>
+#include <Xm/Protocols.h>
+
 #if XmVERSION >= 2
 #include <Xm/CSText.h>
 #endif
@@ -86,6 +91,61 @@ Tm_AnyWidgetCmd(clientData, interp, argc, argv)
     }
     c = argv[1][0];
 
+    if((c=='w') && 0 ==  strcmp(argv[1],"windowId")) {
+      sprintf(interp->result,"0x%lx",(long)XtWindow(w));
+      return TCL_OK;
+    } else
+
+    if((c=='d') && 0 == strcmp(argv[1],"displayInfo")) {
+      Display *d = wPtr->displayInfo->display;
+      sprintf(interp->result,"%.80s %d %d %d %d %d %d %lx",
+              XDisplayString(d),
+              (int)DisplayWidth(d,DefaultScreen(d)),
+              (int)DisplayHeight(d,DefaultScreen(d)),
+              (int)DisplayWidthMM(d,DefaultScreen(d)),
+              (int)DisplayHeightMM(d,DefaultScreen(d)),
+              (int)DefaultScreen(d),
+              (int)DefaultDepth(d,DefaultScreen(d)),
+              (long)DefaultRootWindow(d));
+      return TCL_OK;
+    } else
+
+    if((c == 'l') && (strcmp(argv[1], "lowerWindow") == 0)) {
+	XLowerWindow(XtDisplay(w), XtWindow(w));
+	return TCL_OK;
+    } else
+      
+    if((c == 'r') && (strcmp(argv[1], "raiseWindow") == 0)) {
+	XRaiseWindow(XtDisplay(w), XtWindow(w));
+	return TCL_OK;
+    } else
+    if((c == 'w') && (strcmp(argv[1], "warpPointer") == 0)) {
+	int xpos, ypos;
+
+	if (argc < 4) {
+	    sprintf(interp->result,"warpPointer: wrong # of args, should be 4.");
+	    return TCL_ERROR;
+	}
+	xpos = strtol(argv[2],0,0);
+	ypos = strtol(argv[3],0,0);
+	XWarpPointer(XtDisplay(w),None,XtWindow(w),0,0,0,0,xpos,ypos);
+	return TCL_OK;
+    } else
+    if((c == 'b') && (strcmp(argv[1], "beep") == 0)) {
+	int percent;
+
+	if (argc < 3) {
+	    sprintf(interp->result, "beep: wrong # args, should be 3.");
+	    return TCL_ERROR;
+	}
+	percent = strtol(argv[2],0,0);
+	if(percent < -100)
+	    percent = -100;
+	else if(percent > 100)
+	    percent = 100;
+	XBell(XtDisplay(w),percent);
+	return TCL_OK;
+    } else
 
     if ((c == 'c') && (strcmp(argv[1], "callActionProc") == 0)) {
 	XEvent xev;
@@ -130,6 +190,11 @@ action", argv[0]);
 
     if ((c == 'd') && (strcmp(argv[1], "defineCursor") == 0)) {
 	XrmValue from, to;
+#if 1
+	XrmValue disp;
+	Display *d;
+	Cardinal num = 1;
+#endif
 	Cursor cursor;
 
 	if (argc < 3) {
@@ -147,6 +212,8 @@ value", argv[0]);
 	/* special case: "" restores default */
 	if (argv[2][0] == '\0') {
 	    XDefineCursor(XtDisplay(w), XtWindow(w), None);
+            /* make sure it changes *now* */
+            XFlush(XtDisplay(w));
 	    return TCL_OK;
 	}
 
@@ -155,20 +222,31 @@ value", argv[0]);
 	to.size = sizeof(Cursor *);
 	to.addr = (caddr_t) &cursor;
 
-	if (XtConvertAndStore(w, XmRString, &from,
-				XmRCursor, &to)) {
+#if 1
+	d = XtDisplay(w);
+	disp.size = sizeof(Display*);
+	disp.addr = (XtPointer)&d;
+	if (XtCvtStringToCursor(XtDisplay(w),&disp,&num,&from,&to,NULL))
+#else
+	if (XtConvertAndStore(w,XmRString,&from,XmRCursor, &to))
+#endif
+	{
 	    XDefineCursor(XtDisplay(w), XtWindow(w),* (Cursor *) to.addr);
+            /* make sure it changes *now* */
+            XFlush(XtDisplay(w));
 	} else {
-	    sprintf(interp->result, "wrong cursor to \"%.50s defineCursor \
-value", argv[0]);
+	    sprintf(interp->result, "wrong cursor \"%.50s\" to defineCursor \
+value", argv[2]);
 	    return TCL_ERROR;
 	}
     } else
 
     if ((c == 'd') && (strcmp(argv[1], "destroyWidget") == 0)) {
 	/* destroy - most stuff done in DestroyWidgetHandler */
-
 	XtDestroyWidget(wPtr->widget);
+
+        /* must delete tcl command now, though, so we can't use it again */
+        Tcl_DeleteCommand(interp, wPtr->pathName);
     } else
 
 #ifndef MOTIF11
@@ -216,7 +294,7 @@ resourceList\"", argv[0]);
 
     if ((c == 'g') && (strcmp(argv[1], "getGC") == 0)) {
 	if (argc < 4) {
-	    sprintf(interp->result, "wrong # args: should be \"%.50s setValues \
+	    sprintf(interp->result, "wrong # args: should be \"%.50s getGC \
 -option value...", argv[0]);
 	    return TCL_ERROR;
 	}
@@ -225,6 +303,30 @@ resourceList\"", argv[0]);
 				XtClass(wPtr->widget), argv+2, argc-2);
     } else
 
+    if ((c == 'g') && (strcmp(argv[1], "getSelectionValue") == 0)) {
+	Atom selection;
+	Tm_ClientData *client_data;
+
+        if (argc < 4) {
+            sprintf(interp->result, "wrong # args: should be \"%.50s getSelectionValue \
+atom command", argv[0]);
+            return TCL_ERROR;
+        }
+
+	/* store the widget info and tcl command for the selection callback */
+	client_data = (Tm_ClientData *) XtMalloc(sizeof(Tm_ClientData));
+	client_data->callback_func = argv[3];
+	client_data->widget_info = wPtr;
+
+	selection = XInternAtom(XtDisplay(w), argv[2], False);
+
+	/* we should probably query about possible targets first */
+	XtGetSelectionValue(w, selection, XA_STRING, 
+		Tm_SelectionCallbackHandler, client_data,
+		XtLastTimestampProcessed(XtDisplay(w)));
+
+    } else
+
     if ((c == 'g') && (strcmp(argv[1], "getValues") == 0)) {
 	if (argc < 4) {
 	    sprintf(interp->result, "wrong # args: should be \"%.50s getValues \
@@ -262,6 +364,24 @@ resourceList\"", argv[0]);
 	}
     } else
 
+    if ((c == 'i') && (strcmp(argv[1], "isMapped") == 0)) {
+	XWindowAttributes xwa;
+	if (XGetWindowAttributes(XtDisplay(w), XtWindow(w), &xwa)) {
+	    if (xwa.map_state == IsUnmapped) {
+		Tcl_SetResult(interp, "IsUnmapped", TCL_STATIC);
+	    } else 
+	    if (xwa.map_state == IsUnviewable) {
+		Tcl_SetResult(interp, "IsUnviewable", TCL_STATIC);
+	    } else
+	    if (xwa.map_state == IsViewable) {
+		Tcl_SetResult(interp, "IsViewable", TCL_STATIC);
+	    }
+	} else {
+	    Tcl_SetResult(interp, "Unable to get WindowAttributes", TCL_STATIC);
+	    return TCL_ERROR;
+	}
+    } else
+
     if ((c == 'i') && (strcmp(argv[1], "isSensitive") == 0)) {
 	if (XtIsSensitive(wPtr->widget)) {
 	    interp->result = "true";
@@ -289,39 +409,13 @@ resourceList\"", argv[0]);
 
 #if USE_UIL
     if ((c == 'm') && (strcmp(argv[1], "mrmFetchWidget") == 0)) {
-	Widget wid;
-	MrmType cl;
-	char *path;
+	return Tm_MrmFetchWidget(clientData, interp, argc, argv);
 
-	if (argc < 3) {
-	    sprintf(interp->result, "wrong # args: should be \"%.50s mrmFetchWidget \
-widget", argv[0]);
-	    return TCL_ERROR;
-	}
+    } else
 
-	if (MrmFetchWidget(wPtr->displayInfo->hierarchy, argv[2],
-		w, &wid, &cl) != MrmSUCCESS) {
-	    sprintf(interp->result, "failed to fetch widget %s",
-		argv[2]);
-	    return TCL_ERROR;
-	}
+    if ((c == 'm') && (strcmp(argv[1], "mrmFetchWidgetOverride") == 0)) {
+	return Tm_MrmFetchWidgetOverride(clientData, interp, argc, argv);
 
-	/* make this and its children into Tm widgets */
-	path = XtMalloc(strlen(argv[0]) + strlen(argv[2]) + 2);
-	strcpy(path, argv[0]);
-	/* special case root widget `.' */
-	if (strcmp(argv[0], ".") != 0) {
-	    strcat(path, ".");
-	}
-	strcat(path, argv[2]);
-	Tm_CreateTmInfo(interp, wid, path, argv[0], wPtr->displayInfo);
-	if (XtIsComposite(wid)) {
-	    Tm_CreateTmInfoInChildren(interp, wid, path,
-			wPtr->displayInfo);
-	}
-	XtFree(path);
-	
-	return TCL_OK;
     } else
 #endif /* USE_UIL */
 
@@ -468,7 +562,7 @@ callback command...", argv[0]);
 	}
 
 	/* now look to see if we can find a match */
-	while (callback_list != NULL) {
+	while (callback_list != NULL && callback_list->callback != NULL) {
 	    if (callback_list->callback == Tm_WidgetCallbackHandler) {
 		client_data = (Tm_ClientData *) (callback_list->closure);
 		if (strcmp(callback_func, client_data->callback_func)
@@ -491,6 +585,59 @@ callback command...", argv[0]);
 	XtFree(callback_func);
     } else
 
+    /* translation table stuff */
+    if ((c == 'a') && (strcmp(argv[1], "augmentTranslations") == 0)) {
+	XtTranslations trans;
+
+        if (argc != 3) {
+            sprintf(interp->result,
+                "wrong # args: should be \"%.50s augmentTranslations trans\"",
+                argv[0]);
+            return TCL_ERROR;
+        }
+	trans = XtParseTranslationTable(argv[2]);
+	XtAugmentTranslations(wPtr->widget, trans);
+
+    } else
+
+    if ((c == 'o') && (strcmp(argv[1], "overrideTranslations") == 0)) {
+	XtTranslations trans;
+
+        if (argc != 3) {
+            sprintf(interp->result,
+                "wrong # args: should be \"%.50s overrideTranslations trans\"",
+                argv[0]);
+            return TCL_ERROR;
+        }
+	trans = XtParseTranslationTable(argv[2]);
+        XtOverrideTranslations(wPtr->widget, trans);
+
+    } else
+
+    if ((c == 'u') && (strcmp(argv[1], "uninstallTranslations") == 0)) {
+	XtUninstallTranslations(wPtr->widget);
+    } else
+
+    if ((c == 's') && (strcmp(argv[1], "setCommand") == 0)) {
+	int sc_argc;
+	char **sc_argv;
+
+        if (argc == 2) {
+	    Widget top = w;
+
+	    while (XtParent(top) != NULL) {
+		top = XtParent(top);
+	    }
+	    XtVaGetValues(top, XmNargv, &sc_argv, XmNargc, &sc_argc, NULL);
+        } else {
+	    sc_argc = argc - 2;
+	    sc_argv = argv + 2;
+	}
+	
+	XSetCommand(XtDisplay(w), XtWindow(w), sc_argv, sc_argc);
+
+    } else
+
 #if XmVERSION >= 2
     /* in Motif 2.0 the UTM functions need special handling
      */
@@ -1054,6 +1201,12 @@ Tm_ListWidgetCmd (clientData, interp, argc, argv)
  *	AppContext - in tclMotif, we only have one of these, so
  *	it is sensible to attach them to `.'.
  *
+ *	xtAppInit has been modefied to allow the appContext to be
+ *	made into a command, also vectored through here. So we check
+ *	on this by looking for `.' or otherwise. This is quite kludgy
+ *	but preserves backward compatability. This info is kept in the
+ *	vbl "weAreWidget"
+ *
  * Results:
  *
  * Side effects:
@@ -1068,10 +1221,11 @@ Tm_RootCmd (clientData, interp, argc, argv)
     int argc;
     char **argv;
 {
-    Tm_Widget *wPtr = (Tm_Widget *) clientData;
+    Tm_Widget *wPtr = NULL;
     Widget w;
     char c;
     XtAppContext appContext;
+    Bool weAreWidget = True;
 
     if (argc < 2) {
 	sprintf(interp->result,
@@ -1080,8 +1234,21 @@ Tm_RootCmd (clientData, interp, argc, argv)
 	return TCL_ERROR;
     }
 
-    w = wPtr -> widget;
-    appContext = XtWidgetToApplicationContext(wPtr->widget);
+    /* 
+     * generally, we are the widget ".".
+     * Sometimes we may be a named application context (when there
+     * are more than one app context). Some commands can only be
+     * used by widgets, not app contexts. This is a kludge
+     */
+    if (strcmp(argv[0], ".") == 0) {
+	wPtr = (Tm_Widget *) clientData;
+        w = wPtr -> widget;
+        appContext = XtWidgetToApplicationContext(wPtr->widget);
+    } else {
+	w = NULL;
+	appContext = (XtAppContext) clientData;
+	weAreWidget = False;
+    }
 
     c = argv[1][0];
 
@@ -1089,8 +1256,6 @@ Tm_RootCmd (clientData, interp, argc, argv)
 	Tm_InputData *input;
 	int filenum;
 	char *end;
-	XtInputId inputId;
-	char ch;
 
 	if (argc < 4) {
 	    sprintf(interp->result, "wrong # args: should be \"%.50s fileID \
@@ -1127,32 +1292,38 @@ permissions tcl_code", argv[0]);
 	    Tcl_SetResult(interp, "addInput: only one of r,w,x", TCL_STATIC);
 	    return TCL_ERROR;
 	}
-	
-	ch = argv[3][0];
-	if (ch != 'r' && ch != 'w' && ch != 'x') {
-	    Tcl_SetResult(interp, "addInput: use only r,w,x as modes", TCL_STATIC);
-	    return TCL_ERROR;
-	}
 
-	/* determine modes */
-	if (ch == 'r') {
-	    inputId = XtAppAddInput(appContext, filenum, (XtPointer) XtInputReadMask,
-			Tm_InputHandler, (XtPointer) input);
-	} else
-	if (ch == 'w') {
-	    inputId = XtAppAddInput(appContext, filenum, (XtPointer) XtInputWriteMask,
-			Tm_InputHandler, (XtPointer) input);
-	} else
-	if (ch == 'x') {
-	    inputId = XtAppAddInput(appContext, filenum, (XtPointer) XtInputExceptMask,
-			Tm_InputHandler, (XtPointer) input);
-	}
-	
-	/* simple type check: prefix Id with "inputId-" */
-	sprintf(interp->result, "inputId-%lu", inputId);
+	{
+	    XtInputId inputId;
+	    XtPointer p = NULL;
+	    
+	    switch(argv[3][0]) {
+	      case 'r':
+	      case 'R':
+		p = (XtPointer)XtInputReadMask;
+		break;
+	      case 'w':
+	      case 'W':
+		p = (XtPointer)XtInputWriteMask;
+		break;
+	      case 'x':
+	      case 'X':
+		p = (XtPointer)XtInputExceptMask;
+		break;
+	      default:
+		Tcl_SetResult(interp, "addInput: use only r,w,x as modes",
+			      TCL_STATIC);
+		return TCL_ERROR;
+		break;
+	    }
+	    inputId = XtAppAddInput(appContext,filenum,p,
+				    Tm_InputHandler,(XtPointer)input);
+	    /* simple type check: prefix Id with "inputId-" */
+	    sprintf(interp->result, "inputId-%lu", inputId);
 #	ifdef DEBUG
 	    fprintf(stderr, "adding input %lu\n", inputId);
 #	endif
+	}
 	return TCL_OK;
     } else
 
@@ -1180,12 +1351,14 @@ permissions tcl_code", argv[0]);
 	return TCL_OK;
     } else
 
+
     if ((c == 'm') && (strcmp(argv[1], "mainLoop") == 0)) {
-        XtAppMainLoop(XtWidgetToApplicationContext(wPtr->widget));
+        XtAppMainLoop(appContext);
     } else
 
 #if USE_UIL
-    if ((c == 'm') && (strcmp(argv[1], "mrmCloseHierarchy") == 0)) {
+    if (weAreWidget && (c == 'm') && 
+	(strcmp(argv[1], "mrmCloseHierarchy") == 0)) {
 	if (MrmCloseHierarchy(wPtr->displayInfo->hierarchy) != MrmSUCCESS) {
 	    sprintf(interp->result, "failed to close Mrm hierachy");
 	    return TCL_ERROR;
@@ -1194,7 +1367,8 @@ permissions tcl_code", argv[0]);
     } else
 #endif /* USE_UIL */
 
-    if ((c == 'm') && (strcmp(argv[1], "mrmOpenHierarchy") == 0)) {
+    if (weAreWidget && (c == 'm') && 
+	(strcmp(argv[1], "mrmOpenHierarchy") == 0)) {
 	int largc;
 	char **largv;
 #if USE_UIL
@@ -1224,6 +1398,25 @@ permissions tcl_code", argv[0]);
 
     } else
 
+    if ((c == 'p') && (strcmp(argv[1], "pending") == 0)) {
+	XtInputMask pending;
+	char *pend[3];
+	int n = 0;
+
+	pending = XtAppPending(appContext);
+	if (pending & XtIMXEvent) {
+	    pend[n++] = "XEvent";
+	}
+	if (pending & XtIMTimer) {
+	    pend[n++] = "Timer";
+	}
+	if (pending & XtIMAlternateInput) {
+	     pend[n++] = "AlternateInput";
+	}
+
+	Tcl_SetResult(interp, Tcl_Merge(n, pend), TCL_DYNAMIC);
+    } else
+
     if ((c == 'p') && (strcmp(argv[1], "processEvent") == 0)) {
 	XtAppProcessEvent(appContext, XtIMAll);
     } else
@@ -1264,7 +1457,11 @@ timerID\"", argv[0]);
 	XtRemoveTimeOut(timerId);
 
     } else {
-        return Tm_ShellWidgetCmd(clientData, interp, argc, argv);
+	if (weAreWidget) {
+            return Tm_ShellWidgetCmd(clientData, interp, argc, argv);
+	} else {
+	    return TCL_ERROR;
+	}
     }
     return TCL_OK;
 }
@@ -1352,8 +1549,6 @@ Tm_DrawnWidgetCmd (clientData, interp, argc, argv)
 	int angle1, angle2;
 	Window win;
 	Display *display;
-	int length;
-	char *string;
 
         if (argc != 9) {
     	    sprintf(interp->result,
@@ -1385,8 +1580,6 @@ Tm_DrawnWidgetCmd (clientData, interp, argc, argv)
 	int x1, y1, x2, y2;
 	Window win;
 	Display *display;
-	int length;
-	char *string;
 
         if (argc != 7) {
     	    sprintf(interp->result,
@@ -1646,7 +1839,7 @@ Tm_TextWidgetCmd (clientData, interp, argc, argv)
 #ifndef MOTIF11
     if (strcmp(argv[1], "getSubstring") == 0) {
 	XmTextPosition start;
-	int num_chars;
+	long num_chars;
 	int buffer_size;
 	char *buffer;
 	int ret;
@@ -2638,22 +2831,23 @@ Tm_PopupMenuWidgetCmd (clientData, interp, argc, argv)
 {
     Tm_Widget *wPtr = (Tm_Widget *) clientData;
     Widget w = wPtr->widget;
-    XButtonPressedEvent *event;
 
     if (strcmp(argv[1], "menuPosition") == 0) {
+	unsigned long v;
+	
         if (argc != 3) {
             sprintf(interp->result,
                 "wrong # args: should be \"%.50s menuPosition event\"",
                 argv[0]);
             return TCL_ERROR;
         }
-	if (sscanf(argv[2], "event-%lu", &event) < 1) {
+	if (sscanf(argv[2], "event-%lu", &v) < 1) {
             sprintf(interp->result,
                 "wrong event arg: should be \"%.50s menuPosition event\"",
                 argv[0]);
             return TCL_ERROR;
 	}
-	XmMenuPosition(w, event);
+	XmMenuPosition(w,(XButtonPressedEvent*)v);
 	return TCL_OK;
     } else {
         return Tm_AnyWidgetCmd(clientData, interp, argc, argv);
@@ -2697,9 +2891,37 @@ Tm_ShellWidgetCmd (clientData, interp, argc, argv)
 	XtPopup(w, XtGrabNone);
 	return TCL_OK;
     } else
+
     if (strcmp(argv[1], "popdown") == 0) {
 	XtPopdown(w);
 	return TCL_OK;
+    } else
+
+    if (strcmp(argv[1], "addWMProtocolCallback") == 0) {
+	Tm_ClientData *client_data;
+	Atom wm_atom;
+
+	if (argc < 4) {
+	    sprintf(interp->result,
+			"wrong # args: should be \"%.50s addWMProtocolCallback atom command... \"",
+			argv[0]);
+	    return TCL_ERROR;
+	}
+
+	wm_atom = XmInternAtom(XtDisplay(w), argv[2], False);
+
+	client_data = (Tm_ClientData *) XtMalloc(sizeof(Tm_ClientData));
+
+	if (argc == 4) {
+	    client_data->callback_func = XtNewString(argv[3]);
+	} else {
+	    client_data->callback_func = Tcl_Merge(argc - 3, argv + 3);
+	}
+	client_data->widget_info = wPtr;
+	
+	XmAddWMProtocolCallback(w, wm_atom, 
+			Tm_WidgetCallbackHandler,
+			client_data);
     } else {
         return Tm_AnyWidgetCmd(clientData, interp, argc, argv);
     }
