@@ -69,14 +69,18 @@ static Tm_ReasonType Tm_Reasons[] = {
 	    {XmCR_MULTIPLE_SELECT,		"multiple_select"},
 	    {XmCR_NONE,				"none"},
 	    {XmCR_NO_MATCH,			"no_match"},
+#ifndef MOTIF11
 	    {XmCR_OBSCURED_TRAVERSAL,		"obscured_traversal"},
+#endif /* MOTIF11 */
 	    {XmCR_OK,				"ok"},
 	    {XmCR_PAGE_DECREMENT,		"page_decrement"},
 	    {XmCR_PAGE_INCREMENT,		"page_increment"},
 	    {XmCR_RESIZE,			"resize"},
 	    {XmCR_SINGLE_SELECT,		"single_select"},
+#ifndef MOTIF11
 	    {XmCR_TEAR_OFF_ACTIVATE,		"tear_off_activate"},
 	    {XmCR_TEAR_OFF_DEACTIVATE,		"tear_off_deactivate"},
+#endif /* MOTIF11 */
 	    {XmCR_TO_BOTTOM,			"to_bottom"},
 	    {XmCR_TO_TOP,			"to_top"},
 	    {XmCR_UNMAP,			"unmap"},
@@ -186,17 +190,15 @@ items2String(count, selected_items)
     int count;
     XmString *selected_items;
 {
-        String *items;
 	static String items_list = NULL;
 	static int items_list_len = 0;
+        String *items;
 	int n, len;
-	char *p, *end;
 
 	items = (char **) XtMalloc(sizeof(char *) * count);
 	len= 0;
 	for (n = 0; n < count; n++) {
-	    items[n] = XtNewString(
-			Tm_XmStringToString(selected_items[n]));
+	    items[n] = XtNewString(Tm_XmStringToString(selected_items[n]));
 	    len += strlen(items[n]) + 2; /* for , */
 	}
 
@@ -235,7 +237,8 @@ items2String(count, selected_items)
 	return items_list;
 }
 
- 
+
+#if XmVERSION >= 2
 /*
  *--------------------------------------------------------------
  *
@@ -258,18 +261,16 @@ widgets2String(count, w_items)
     int count;
     WidgetList w_items;
 {
-    String *items;
     static String items_list = NULL;
     static int items_list_len = 0;
+    String *items;
     int n, len;
-    char *p, *end;
     Tm_Widget *wPtr;
 
     items = (char **) XtMalloc(sizeof(char *) * count);
     len= 0;
     for (n = 0; n < count; n++) {
-    	XtVaGetValues(w_items[n], 
-			XmNuserData, &wPtr, NULL);
+    	XtVaGetValues(w_items[n],XmNuserData, &wPtr, NULL);
         items[n] = wPtr->pathName;
         len += strlen(items[n]) + 2; /* for , */
     }
@@ -296,6 +297,7 @@ widgets2String(count, w_items)
     return items_list;
 	    
 }
+#endif
 
 /*
  *--------------------------------------------------------------
@@ -323,7 +325,6 @@ getValue(pathName, w, event, call_data, value, length)
     int length;
 {
     char *result;
-    XrmValue from, to;
     static char buf[32];
     static char event_buf[32];
     static char *reason;
@@ -331,13 +332,10 @@ getValue(pathName, w, event, call_data, value, length)
     XmAnyCallbackStruct *c_data = (XmAnyCallbackStruct *) call_data;
     char *all[32];	/* max elmts in callback struct */
     int n_all = 0;
-    static char x[32], y[32];
     int n;
 
-    if (length == strlen("w") &&
-	    strncmp(value, "w", length) == 0) {
+    if (length == 1 && *value == 'w')
 	return pathName;
-    }
 
     if (length == strlen("dragContext") &&
 	    strncmp(value, "dragContext", length) == 0) {
@@ -374,71 +372,9 @@ getValue(pathName, w, event, call_data, value, length)
 	/* nowhere to be found */
 	reason = "none";
     }
-#if 0
-    switch (c_data->reason) {
-	    case XmCR_ACTIVATE:			reason = "activate"; 	break;
-	    case XmCR_APPLY:			reason = "apply"; 	break;
-	    case XmCR_ARM:			reason = "arm"; 	break;
-	    case XmCR_BROWSE_SELECT:		reason = "browse_select"; 	break;
-	    case XmCR_CANCEL:			reason = "cancel"; 	break;
-	    case XmCR_CASCADING:		reason = "cascading"; 	break;
-	    case XmCR_CLIPBOARD_DATA_DELETE:	reason = "clipboard_data_delete"; 	break;
-	    case XmCR_CLIPBOARD_DATA_REQUEST:	reason = "clipboard_data_request"; 	break;
-	    case XmCR_COMMAND_CHANGED:		reason = "command_changed"; 	break;
-	    case XmCR_COMMAND_ENTERED:		reason = "command_entered"; 	break;
-	    case XmCR_CREATE:			reason = "create"; 	break;
-	    case XmCR_DECREMENT:		reason = "decrement"; 	break;
-	    case XmCR_DEFAULT_ACTION:		reason = "default_action"; 	break;
-	    case XmCR_DISARM:			reason = "disarm"; 	break;
-	    case XmCR_DRAG:			reason = "drag"; 	break;
-	    case XmCR_EXECUTE:			reason = "execute"; 	break;
-	    case XmCR_EXPOSE:			reason = "expose"; 	break;
-	    case XmCR_EXTENDED_SELECT:		reason = "extended_select"; 	break;
-	    case XmCR_FOCUS:			reason = "focus"; 	break;
-	    case XmCR_GAIN_PRIMARY:		reason = "gain_primary"; 	break;
-	    case XmCR_HELP:			reason = "help"; 	break;
-	    case XmCR_INCREMENT:		reason = "increment"; 	break;
-	    case XmCR_INPUT:			reason = "input"; 	break;
-	    case XmCR_LOSE_PRIMARY:		reason = "lose_primary"; 	break;
-	    case XmCR_LOSING_FOCUS:		reason = "losing_focus"; 	break;
-	    case XmCR_MAP:			reason = "map"; 	break;
-	    case XmCR_MODIFYING_TEXT_VALUE:	reason = "modifying_text_value"; 	break;
-	    case XmCR_MOVING_INSERT_CURSOR:	reason = "moving_insert_cursor"; 	break;
-	    case XmCR_MULTIPLE_SELECT:		reason = "multiple_select"; 	break;
-	    case XmCR_NONE:			reason = "none"; 	break;
-	    case XmCR_NO_MATCH:			reason = "no_match"; 	break;
-	    case XmCR_OBSCURED_TRAVERSAL:	reason = "obscured_traversal"; 	break;
-	    case XmCR_OK:			reason = "ok"; 	break;
-	    case XmCR_PAGE_DECREMENT:		reason = "page_decrement"; 	break;
-	    case XmCR_PAGE_INCREMENT:		reason = "page_increment"; 	break;
-	    case XmCR_RESIZE:			reason = "resize"; 	break;
-	    case XmCR_SINGLE_SELECT:		reason = "single_select"; 	break;
-	    case XmCR_TEAR_OFF_ACTIVATE:	reason = "tear_off_activate"; 	break;
-	    case XmCR_TEAR_OFF_DEACTIVATE:	reason = "tear_off_deactivate"; 	break;
-	    case XmCR_TO_BOTTOM:		reason = "to_bottom"; 	break;
-	    case XmCR_TO_TOP:			reason = "to_top"; 	break;
-	    case XmCR_UNMAP:			reason = "unmap"; 	break;
-	    case XmCR_VALUE_CHANGED:		reason = "value_changed"; 	break;
-#if XmVersion >= 2000
-	    case XmCR_COLLAPSED:		reason = "collapsed"; 	break;
-	    case XmCR_EXPANDED:			reason = "expanded"; 	break;
-	    case XmCR_MAJOR_TAB:		reason = "major_tab"; 	break;
-	    case XmCR_MINOR_TAB:		reason = "minor_tab"; 	break;
-	    case XmCR_PAGE_SCROLLER_INCREMENT:	reason = "page_scroller_increment"; 	break;
-	    case XmCR_SPIN_NEXT:		reason = "spin_next"; 	break;
-	    case XmCR_SPIN_PRIOR:		reason = "spin_prior"; 	break;
-	    case XmCR_SPIN_FIRST:		reason = "spin_first"; 	break;
-	    case XmCR_SPIN_LAST:		reason = "spin_last"; 	break;
-	    case XmCR_SELECT:			reason = "select"; 	break;
-#endif
-	    default: 				reason = "none";
-    }
-#endif /* 0 */
 
-    if (length == strlen("reason") &&
-	    strncmp(value, "reason", length) == 0) {
+    if (length == strlen("reason") && strncmp(value, "reason", length) == 0)
 	return reason;
-    }
 
     if (length == strlen("call_data") &&
 	    strncmp(value, "call_data", length) == 0) {
@@ -451,16 +387,15 @@ getValue(pathName, w, event, call_data, value, length)
 
     class = XtClass(w);
     if (XtIsSubclass(w, xmArrowButtonWidgetClass)) {
-	XmArrowButtonCallbackStruct *c_data = (XmArrowButtonCallbackStruct *) call_data;
+	XmArrowButtonCallbackStruct *c_data =
+	    (XmArrowButtonCallbackStruct *) call_data;
 
 	if (length == strlen("click_count") &&
-		strncmp(value, "click_count", length) == 0) {
+	    strncmp(value, "click_count", length) == 0) {
 	    sprintf(buf, "%d", c_data->click_count);
 	    return buf;
-	} else
-
-	if (length == strlen("call_data") &&
-		strncmp(value, "call_data", length) == 0) {
+	} else if (length == strlen("call_data") &&
+		   strncmp(value, "call_data", length) == 0) {
 	    if (c_data->reason == XmCR_ACTIVATE) {
 	        sprintf(buf, "%d", c_data->click_count);
 	        all[n_all++] = make2list("click_count", buf);
@@ -468,36 +403,26 @@ getValue(pathName, w, event, call_data, value, length)
 	    } else {
 		return Tm_Merge(n_all, all);
 	    }
-
 	} else {
 	    return "ERROR in substitution";
 	}
-    } else
-
-    if (XtIsSubclass(w, xmCommandWidgetClass)) {
+    } else if (XtIsSubclass(w, xmCommandWidgetClass)) {
 	XmCommandCallbackStruct *c_data = (XmCommandCallbackStruct *) call_data;
 
 	if (length == strlen("value") &&
-		strncmp(value, "value", length) == 0) {
+	    strncmp(value, "value", length) == 0) {
 	    result = Tm_XmStringToString(c_data->value);
 	    return result;
-	} else
-
-	if (length == strlen("length") &&
-		strncmp(value, "length", length) == 0) {
+	} else if (length == strlen("length") &&
+		   strncmp(value, "length", length) == 0) {
 	    sprintf(buf, "%d", c_data->length);
 	    return buf;
-	} else
-
-	if (length == strlen("call_data") &&
-		strncmp(value, "call_data", length) == 0) {
+	} else if (length == strlen("call_data") &&
+		   strncmp(value, "call_data", length) == 0) {
 	    all[n_all++] = make2list("value", Tm_XmStringToString(c_data->value));
-
 	    sprintf(buf, "%d", c_data->length);
 	    all[n_all++] = make2list("length", buf);
-
 	    return Tm_Merge(n_all, all);
-
 	} else {
 	    return "ERROR in substitution";
 	}
@@ -528,79 +453,59 @@ getValue(pathName, w, event, call_data, value, length)
 #endif
 
     if (XtIsSubclass(w, xmDrawnButtonWidgetClass)) {
-	XmDrawnButtonCallbackStruct *c_data = (XmDrawnButtonCallbackStruct *) call_data;
+	XmDrawnButtonCallbackStruct *c_data =
+	    (XmDrawnButtonCallbackStruct *) call_data;
 
 	if (length == strlen("click_count") &&
-		strncmp(value, "click_count", length) == 0) {
+	    strncmp(value, "click_count", length) == 0) {
 	    sprintf(buf, "%d", c_data->click_count);
 	    return buf;
-	} else
-
-	if (length == strlen("call_data") &&
-		strncmp(value, "call_data", length) == 0) {
+	} else if (length == strlen("call_data") &&
+		   strncmp(value, "call_data", length) == 0) {
 	    sprintf(buf, "%d", c_data->click_count);
 	    all[n_all++] = make2list("click_count", buf);
 	    return Tm_Merge(n_all, all);
-
 	} else {
 	    return "ERROR in substitution";
 	}
-    } else
-
-    if (XtIsSubclass(w, xmFileSelectionBoxWidgetClass)) {
+    } else if (XtIsSubclass(w, xmFileSelectionBoxWidgetClass)) {
 	XmFileSelectionBoxCallbackStruct *c_data =
-			(XmFileSelectionBoxCallbackStruct *) call_data;
-
+	    (XmFileSelectionBoxCallbackStruct *) call_data;
+	
 	if (length == strlen("value") &&
-		strncmp(value, "value", length) == 0) {
+	    strncmp(value, "value", length) == 0) {
 	    result = Tm_XmStringToString(c_data->value);
 	    return result;
-	} else
-
-	if (length == strlen("length") &&
-		strncmp(value, "length", length) == 0) {
+	} else if (length == strlen("length") &&
+		   strncmp(value, "length", length) == 0) {
 	    sprintf(buf, "%d", c_data->length);
 	    return buf;
-	} else
-
-	if (length == strlen("mask") &&
-		strncmp(value, "mask", length) == 0) {
+	} else if (length == strlen("mask") &&
+		   strncmp(value, "mask", length) == 0) {
 	    result = Tm_XmStringToString(c_data->mask);
 	    return result;
-	} else
-
-	if (length == strlen("mask_length") &&
-		strncmp(value, "mask_length", length) == 0) {
+	} else if (length == strlen("mask_length") &&
+		   strncmp(value, "mask_length", length) == 0) {
 	    sprintf(buf, "%d", c_data->mask_length);
 	    return buf;
-	} else
-
-	if (length == strlen("dir") &&
-		strncmp(value, "dir", length) == 0) {
+	} else if (length == strlen("dir") &&
+		   strncmp(value, "dir", length) == 0) {
 	    result = Tm_XmStringToString(c_data->dir);
 	    return result;
-	} else
-
-	if (length == strlen("dir_length") &&
-		strncmp(value, "dir_length", length) == 0) {
+	} else if (length == strlen("dir_length") &&
+		   strncmp(value, "dir_length", length) == 0) {
 	    sprintf(buf, "%d", c_data->dir_length);
 	    return buf;
-	} else
-
-	if (length == strlen("pattern") &&
-		strncmp(value, "pattern", length) == 0) {
-	   result = Tm_XmStringToString(c_data->pattern);
+	} else if (length == strlen("pattern") &&
+		   strncmp(value, "pattern", length) == 0) {
+	    result = Tm_XmStringToString(c_data->pattern);
 	    return result;
-	} else
-
-	if (length == strlen("pattern_length") &&
-		strncmp(value, "pattern_length", length) == 0) {
+	} else if (length == strlen("pattern_length") &&
+		   strncmp(value, "pattern_length", length) == 0) {
 	    sprintf(buf, "%d", c_data->pattern_length);
 	    return buf;
-	} else
-
-	if (length == strlen("call_data") &&
-		strncmp(value, "call_data", length) == 0) {
+	} else if (length == strlen("call_data") &&
+		   strncmp(value, "call_data", length) == 0) {
 	    all[n_all++] = make2list("value", Tm_XmStringToString(c_data->value));
 
 	    sprintf(buf, "%d", c_data->length);
@@ -625,102 +530,46 @@ getValue(pathName, w, event, call_data, value, length)
 	} else {
 	    return "ERROR in substitution";
 	}
-    } else
-
-    if (XtIsSubclass(w, xmListWidgetClass)) {
+    } else if (XtIsSubclass(w, xmListWidgetClass)) {
 	XmListCallbackStruct *c_data = (XmListCallbackStruct *) call_data;
-
-	if (length == strlen("item") &&
-		strncmp(value, "item",length) == 0) {
+	
+	if (length == strlen("item") && strncmp(value, "item",length) == 0) {
 	    result = Tm_XmStringToString(c_data->item);
 	    return result;
-	} else
-	if (length == strlen("item_length") &&
-		strncmp(value, "item_length", length) == 0) {
+	} else if (length == strlen("item_length") &&
+		   strncmp(value, "item_length", length) == 0) {
 	    sprintf(buf, "%d", c_data->item_length);
 	    return buf;
-	} else
-	if (length == strlen("item_position") &&
-		strncmp(value, "item_position", length) == 0) {
+	} else if (length == strlen("item_position") &&
+		   strncmp(value, "item_position", length) == 0) {
 	    sprintf(buf, "%d", c_data->item_position);
 	    return buf;
-	} else
-	if (length == strlen("selected_items") &&
-		strncmp(value, "selected_items", length) == 0) {
+	} else if (length == strlen("selected_items") &&
+		   strncmp(value, "selected_items", length) == 0) {
 	    if (c_data->reason == XmCR_MULTIPLE_SELECT ||
 	    	c_data->reason == XmCR_EXTENDED_SELECT) {
 
 		return items2String(c_data->selected_item_count,
 				    c_data->selected_items);
-#if 0
-		String *items;
-		static String items_list = NULL;
-		static int items_list_len = 0;
-		int n, len;
-		char *p, *end;
-
-		items = (char **) XtMalloc(sizeof(char *) *
-					c_data->selected_item_count);
-		len= 0;
-		for (n = 0; n < c_data->selected_item_count; n++) {
-		    items[n] = XtNewString(
-				Tm_XmStringToString(c_data->selected_items[n]));
-		    len += strlen(items[n]) + 2; /* for , */
-		}
-
-		/* reuse the static space we malloc for this list */
-		if (items_list == NULL) {
-		    items_list = XtMalloc(len + 1); /* we overshoot by 1 */
-		    items_list_len = len + 1;
-		} else
-		if (items_list_len < len) {
-		   items_list = XtRealloc(items_list, len + 1);
-		   items_list_len = len + 1;
-		}
-
-		if (len == 0) {
-		    *items_list = '\0';
-		    return items_list;
-		}
-#ifdef TM_MOTIF_XMSTRINGTABLE
-		end = items_list;
-		for (n = 0; n < c_data->selected_item_count; n++) {
-		    p = items[n];
-		    while (*end++ = *p++)
-			;
-		    *(end - 1) = ',';
-		    *end++ = ' ';
-		    XtFree(items[n]);
-		}
-		*(end - 2) = '\0';
-#else
-		items_list = Tcl_Merge(c_data->selected_item_count, items);
-                for (n = 0; n < c_data->selected_item_count; n++) {
-                    XtFree(items[n]);
-                }
-#endif
-		XtFree((char *) items);
-		return items_list;
-#endif /* 0 */
 	    }
-	} else
-	if (length == strlen("selection_type") &&
-		strncmp(value, "selection_type", length) == 0) {
+	} else if (length == strlen("selection_type") &&
+		   strncmp(value, "selection_type", length) == 0) {
 	    switch (c_data->selection_type) {
-		case XmINITIAL:		return "initial";
-		case XmMODIFICATION:	return "modification";
-		case XmADDITION:	return "addition";
-		default:		return "ERROR!";
+	      case XmINITIAL:
+		return "initial";
+	      case XmMODIFICATION:
+		return "modification";
+	      case XmADDITION:
+		return "addition";
+	      default:
+		return "ERROR!";
 	    }
-	} else
-	if (length == strlen("selected_item_count") &&
-		strncmp(value, "selected_item_count", length) == 0) {
+	} else if (length == strlen("selected_item_count") &&
+		   strncmp(value, "selected_item_count", length) == 0) {
 	    sprintf(buf, "%d", c_data->selected_item_count);
 	    return buf;
-	} else
-
-	if (length == strlen("call_data") &&
-		strncmp(value, "call_data", length) == 0) {
+	} else if (length == strlen("call_data") &&
+		   strncmp(value, "call_data", length) == 0) {
 	    char *type;
 
 	    all[n_all++] = make2list("item",
@@ -751,19 +600,16 @@ getValue(pathName, w, event, call_data, value, length)
 	} else {
 	    return "ERROR in substitution";
 	}
-    } else
-
-    if (XtIsSubclass(w, xmPushButtonWidgetClass)) {
-        XmPushButtonCallbackStruct *c_data = (XmPushButtonCallbackStruct *) call_data;
+    } else if (XtIsSubclass(w, xmPushButtonWidgetClass)) {
+        XmPushButtonCallbackStruct *c_data =
+	    (XmPushButtonCallbackStruct *) call_data;
 
         if (length == strlen("click_count") &&
-                strncmp(value, "click_count", length) == 0) {
+	    strncmp(value, "click_count", length) == 0) {
             sprintf(buf, "%d", c_data->click_count);
             return buf;
-	} else
-
-	if (length == strlen("call_data") &&
-		strncmp(value, "call_data", length) == 0) {
+	} else if (length == strlen("call_data") &&
+		   strncmp(value, "call_data", length) == 0) {
 	    if (c_data->reason == XmCR_ACTIVATE) {
 	        sprintf(buf, "%d", c_data->click_count);
 	        all[n_all++] = make2list("click_count", buf);
@@ -771,96 +617,75 @@ getValue(pathName, w, event, call_data, value, length)
 	    } else {
 		return Tm_Merge(n_all, all);
 	    }
-
         } else {
             return "ERROR in substitution";
 	}
-    } else
-
-    if (XtIsSubclass(w, xmScaleWidgetClass)) {
+    } else if (XtIsSubclass(w, xmScaleWidgetClass)) {
 	XmScaleCallbackStruct *c_data = (XmScaleCallbackStruct *) call_data;
-
+	
 	if (length == strlen("value") &&
-		strncmp(value, "value", length) == 0) {
+	    strncmp(value, "value", length) == 0) {
 	    sprintf(buf, "%d", c_data->value);
 	    return buf;
-	} else
-
-	if (length == strlen("call_data") &&
-		strncmp(value, "call_data", length) == 0) {
+	} else if (length == strlen("call_data") &&
+		   strncmp(value, "call_data", length) == 0) {
 	    sprintf(buf, "%d", c_data->value);
 	    all[n_all++] = make2list("value", buf);
 	    return Tm_Merge(n_all, all);
-
 	} else {
 	    return "ERROR in substitution";
 	}
-    } else
-
-    if (XtIsSubclass(w, xmScrollBarWidgetClass)) {
+    } else if (XtIsSubclass(w, xmScrollBarWidgetClass)) {
 	XmScrollBarCallbackStruct *c_data = (XmScrollBarCallbackStruct *) call_data;
-
+	
 	if (length == strlen("value") &&
 		strncmp(value, "value", length) == 0) {
 	    sprintf(buf, "%d", c_data->value);
 	    return buf;
-	} else
-
-	if (length == strlen("call_data") &&
-		strncmp(value, "call_data", length) == 0) {
+	} else if (length == strlen("call_data") &&
+		   strncmp(value, "call_data", length) == 0) {
 	    sprintf(buf, "%d", c_data->value);
 	    all[n_all++] = make2list("value", buf);
 	    return Tm_Merge(n_all, all);
-
 	} else {
 	    return "ERROR in substitution";
 	}
-    } else
-
-    if (XtIsSubclass(w, xmTextWidgetClass) ||
-	XtIsSubclass(w, xmTextFieldWidgetClass)) {
+    } else if (XtIsSubclass(w, xmTextWidgetClass) ||
+	       XtIsSubclass(w, xmTextFieldWidgetClass)) {
 	XmTextVerifyCallbackStruct *c_data =
-			(XmTextVerifyCallbackStruct *) call_data;
+	    (XmTextVerifyCallbackStruct *) call_data;
 
 	if (length == strlen("doit") &&
 		strncmp(value, "doit", length) == 0) {
 		return TM_TEXT_DOIT;
-	} else
-	if (length == strlen("currInsert") &&
-		strncmp(value, "currInsert", length) == 0) {
-	    sprintf(buf, "%d", c_data->currInsert);
+	} else if (length == strlen("currInsert") &&
+		   strncmp(value, "currInsert", length) == 0) {
+	    sprintf(buf, "%ld", (long)c_data->currInsert);
 	    return buf;
-	} else
-	if (length == strlen("newInsert") &&
-		strncmp(value, "newInsert", length) == 0) {
-	    sprintf(buf, "%d", c_data->newInsert);
+	} else if (length == strlen("newInsert") &&
+		   strncmp(value, "newInsert", length) == 0) {
+	    sprintf(buf, "%ld", (long)c_data->newInsert);
 	    return buf;
-	} else
-	if (length == strlen("startPos") &&
-		strncmp(value, "startPos", length) == 0) {
-		return TM_TEXT_STARTPOS;
-	} else
-	if (length == strlen("endPos") &&
-		strncmp(value, "endPos", length) == 0) {
-		return TM_TEXT_ENDPOS;
-	} else
-	if (length == strlen("ptr") &&
-		strncmp(value, "ptr", length) == 0) {
-		return TM_TEXT_PTR;
-	} else
-	if (length == strlen("length") &&
-		strncmp(value, "length", length) == 0) {
-		return TM_TEXT_LENGTH;
-	} else
-
-	if (length == strlen("call_data") &&
-		strncmp(value, "call_data", length) == 0) {
+	} else if (length == strlen("startPos") &&
+		   strncmp(value, "startPos", length) == 0) {
+	    return TM_TEXT_STARTPOS;
+	} else if (length == strlen("endPos") &&
+		   strncmp(value, "endPos", length) == 0) {
+	    return TM_TEXT_ENDPOS;
+	} else if (length == strlen("ptr") &&
+		   strncmp(value, "ptr", length) == 0) {
+	    return TM_TEXT_PTR;
+	} else if (length == strlen("length") &&
+		   strncmp(value, "length", length) == 0) {
+	    return TM_TEXT_LENGTH;
+	} else if (length == strlen("call_data") &&
+		   strncmp(value, "call_data", length) == 0) {
 	    all[n_all++] = make2list("doit", TM_TEXT_DOIT);
 
-	    sprintf(buf, "%d", c_data->currInsert);
+	    sprintf(buf, "%ld", (long)c_data->currInsert);
 	    all[n_all++] = make2list("currInsert", buf);
 
-	    sprintf(buf, "%d", c_data->newInsert);
+	    sprintf(buf, "%ld", (long)c_data->newInsert);
 	    all[n_all++] = make2list("newInsert", buf);
 
 	    all[n_all++] = make2list("startPos", TM_TEXT_STARTPOS);
@@ -876,61 +701,46 @@ getValue(pathName, w, event, call_data, value, length)
 	} else {
 	    return "ERROR in substitution";
 	}
-    } else
-
-    if (XtIsSubclass(w, xmSelectionBoxWidgetClass)) {
+    } else if (XtIsSubclass(w, xmSelectionBoxWidgetClass)) {
 	XmSelectionBoxCallbackStruct *c_data =
-			(XmSelectionBoxCallbackStruct *) call_data;
+	    (XmSelectionBoxCallbackStruct *) call_data;
 
 	if (length == strlen("value") &&
 		strncmp(value, "value",length) == 0) {
 	    result = Tm_XmStringToString(c_data->value);
 	    return result;
-	} else
-	if (length == strlen("length") &&
-		strncmp(value, "length", length) == 0) {
+	} else if (length == strlen("length") &&
+		   strncmp(value, "length", length) == 0) {
 	    sprintf(buf, "%d", c_data->length);
 	    return buf;
-        } else
-
-        if (length == strlen("call_data") &&
-                strncmp(value, "call_data", length) == 0) {
+        } else if (length == strlen("call_data") &&
+		   strncmp(value, "call_data", length) == 0) {
             all[n_all++] = make2list("value", Tm_XmStringToString(c_data->value));
-
 	    sprintf(buf, "%d", c_data->length);
             all[n_all++] = make2list("value", buf);
-
             return Tm_Merge(n_all, all);
-
-
 	} else {
 	    return "ERROR in substitution";
 	}
-    } else
-
-    if (XtIsSubclass(w, xmToggleButtonWidgetClass)) {
+    } else if (XtIsSubclass(w, xmToggleButtonWidgetClass)) {
 	XmToggleButtonCallbackStruct *c_data =
-			(XmToggleButtonCallbackStruct *) call_data;
+	    (XmToggleButtonCallbackStruct *) call_data;
 
 	if (length == strlen("set") &&
-		strncmp(value, "set", length) == 0) {
+	    strncmp(value, "set", length) == 0) {
 	    if (c_data->set) {
 		return "true";
 	    } else {
 		return "false";
 	    }
-        } else
-
-        if (length == strlen("call_data") &&
-                strncmp(value, "call_data", length) == 0) {
+        } else if (length == strlen("call_data") &&
+		   strncmp(value, "call_data", length) == 0) {
 	    if (c_data->set) {
                 all[n_all++] = make2list("set", "true");
 	    } else {
                 all[n_all++] = make2list("set", "false");
 	    }
- 
             return Tm_Merge(n_all, all);
-
 	} else {
 	    return "ERROR in substitution";
 	}
@@ -1010,55 +820,13 @@ getValue(pathName, w, event, call_data, value, length)
 		strncmp(value, "selected_items", length) == 0) {
 	    return widgets2String(c_data->selected_item_count,
 				  c_data->selected_items);
-#if 0
-	    String *items;
-	    static String items_list = NULL;
-	    static int items_list_len = 0;
-	    int n, len;
-	    char *p, *end;
-    	    Tm_Widget *wPtr;
-
-	    items = (char **) XtMalloc(sizeof(char *) *
-					c_data->selected_item_count);
-	    len= 0;
-	    for (n = 0; n < c_data->selected_item_count; n++) {
-	    	XtVaGetValues(c_data->selected_items[n], 
-				XmNuserData, &wPtr, NULL);
-	        items[n] = wPtr->pathName;
-	        len += strlen(items[n]) + 2; /* for , */
-	    }
-
-	    /* reuse the static space we malloc for this list */
-	    if (items_list == NULL) {
-		    items_list = XtMalloc(len + 1); /* we overshoot by 1 */
-		    items_list_len = len + 1;
-	    } else
-	    if (items_list_len < len) {
-		   items_list = XtRealloc(items_list, len + 1);
-		   items_list_len = len + 1;
-	    }
-
-	    if (len == 0) {
-	        *items_list = '\0';
-	        return items_list;
-	    }
-	    items_list = Tcl_Merge(c_data->selected_item_count, items);
-            for (n = 0; n < c_data->selected_item_count; n++) {
-                XtFree(items[n]);
-            }
-	    XtFree((char *) items);
-	    return items_list;
-#endif /* 0 */
-	    
-	} else
-	if (length == strlen("selected_item_count") &&
+	} else if (length == strlen("selected_item_count") &&
 		strncmp(value, "selected_item_count", length) == 0) {
 	    sprintf(buf, "%d", c_data->selected_item_count);
 	    return buf;
-	} else
-	if (length == strlen("auto_selection_type") &&
-		strncmp(value, "auto_selection_type", length) == 0) {
-	   switch (c_data->auto_selection_type) {
+	} else if (length == strlen("auto_selection_type") &&
+		   strncmp(value, "auto_selection_type", length) == 0) {
+	    switch (c_data->auto_selection_type) {
 		case XmAUTO_UNSET:	return "auto_unset";
 		case XmAUTO_BEGIN:	return "auto_begin";
 		case XmAUTO_MOTION:	return "auto_motion";
@@ -1067,19 +835,16 @@ getValue(pathName, w, event, call_data, value, length)
 		case XmAUTO_CHANGE:	return "auto_change";
 		default:		return "ERROR in substitution";
 	    }
-        } else
-
-        if (length == strlen("call_data") &&
-                strncmp(value, "call_data", length) == 0) {
+        } else if (length == strlen("call_data") &&
+		   strncmp(value, "call_data", length) == 0) {
 	    char *type;
-
+	    
 	    all[n_all++] = make2list("selected_items",
-				 widgets2String(c_data->selected_item_count,
-				  		c_data->selected_items));
+				     widgets2String(c_data->selected_item_count,
+						    c_data->selected_items));
 
 	    sprintf(buf, "%d", c_data->selected_item_count);
             all[n_all++] = make2list("selected_item_count", buf);
-
  	    switch (c_data->auto_selection_type) {
 		case XmAUTO_UNSET:	type = "auto_unset";
 		case XmAUTO_BEGIN:	type = "auto_begin";
@@ -1090,70 +855,55 @@ getValue(pathName, w, event, call_data, value, length)
 		default:		type = "ERROR in substitution";
 	    }
             all[n_all++] = make2list("auto_selection_type", type);
-
             return Tm_Merge(n_all, all);
-
 	} else {
 	    return "ERROR in substitution";
 	}
-    } else
-
-    if (XtIsSubclass(w, xmComboBoxWidgetClass)) {
+    } else if (XtIsSubclass(w, xmComboBoxWidgetClass)) {
 	XmComboBoxCallbackStruct *c_data =
-			(XmComboBoxCallbackStruct *) call_data;
+	    (XmComboBoxCallbackStruct *) call_data;
 
 	if (length == strlen("item_position") &&
-		strncmp(value, "item_position", length) == 0) {
+	    strncmp(value, "item_position", length) == 0) {
 	    sprintf(buf, "%d", c_data->item_position);
 	    return buf;
-	} else
-	if (length == strlen("item_or_text") &&
-		strncmp(value, "item_or_text", length) == 0) {
+	} else if (length == strlen("item_or_text") &&
+		   strncmp(value, "item_or_text", length) == 0) {
 	    return Tm_XmStringToString(c_data->item_or_text);
-        } else
-
-        if (length == strlen("call_data") &&
-                strncmp(value, "call_data", length) == 0) {
+        } else if (length == strlen("call_data") &&
+		   strncmp(value, "call_data", length) == 0) {
             all[n_all++] = make2list("item_position", c_data->item_position);
             all[n_all++] = make2list("item_or_text", Tm_XmStringToString(c_data->item_or_text));
             return Tm_Merge(n_all, all);
-
 	} else {
 	    return "ERROR in substitution";
 	}
-    } else
-
-    if (XtIsSubclass(w, xmNotebookWidgetClass)) {
+    } else if (XtIsSubclass(w, xmNotebookWidgetClass)) {
 	XmNotebookCallbackStruct *c_data =
-			(XmNotebookCallbackStruct *) call_data;
+	    (XmNotebookCallbackStruct *) call_data;
 
 	if (length == strlen("page_number") &&
-		strncmp(value, "page_number", length) == 0) {
+	    strncmp(value, "page_number", length) == 0) {
 	    sprintf(buf, "%d", c_data->page_number);
 	    return buf;
-	} else
-	if (length == strlen("prev_page_number") &&
-		strncmp(value, "prev_page_number", length) == 0) {
+	} else if (length == strlen("prev_page_number") &&
+		   strncmp(value, "prev_page_number", length) == 0) {
 	    sprintf(buf, "%d", c_data->prev_page_number);
 	    return buf;
-	} else
-	if (length == strlen("page_widget") &&
-		strncmp(value, "page_widget", length) == 0) {
+	} else if (length == strlen("page_widget") &&
+		   strncmp(value, "page_widget", length) == 0) {
     	    Tm_Widget *wPtr;
 
 	    XtVaGetValues(c_data->page_widget, XmNuserData, &wPtr, NULL);
 	    return wPtr->pathName;
-	} else
-	if (length == strlen("prev_page_widget") &&
-		strncmp(value, "prev_page_widget", length) == 0) {
+	} else if (length == strlen("prev_page_widget") &&
+		   strncmp(value, "prev_page_widget", length) == 0) {
     	    Tm_Widget *wPtr;
 
 	    XtVaGetValues(c_data->prev_page_widget, XmNuserData, &wPtr, NULL);
 	    return wPtr->pathName;
-        } else
-
-        if (length == strlen("call_data") &&
-                strncmp(value, "call_data", length) == 0) {
+        } else if (length == strlen("call_data") &&
+		   strncmp(value, "call_data", length) == 0) {
     	    Tm_Widget *wPtr;
 
 	    sprintf(buf, "%d", c_data->page_number);
@@ -1173,42 +923,34 @@ getValue(pathName, w, event, call_data, value, length)
 	} else {
 	    return "ERROR in substitution";
 	}
-    } else
-
-    if (XtIsSubclass(w, xmSpinBoxWidgetClass)) {
+    } else if (XtIsSubclass(w, xmSpinBoxWidgetClass)) {
 	XmSpinBoxCallbackStruct *c_data =
-			(XmSpinBoxCallbackStruct *) call_data;
+	    (XmSpinBoxCallbackStruct *) call_data;
 
 	if (length == strlen("position") &&
-		strncmp(value, "position", length) == 0) {
+	    strncmp(value, "position", length) == 0) {
 	    sprintf(buf, "%d", c_data->position);
 	    return buf;
-	} else
-	if (length == strlen("crossed_boundary") &&
-		strncmp(value, "crossed_boundary", length) == 0) {
+	} else if (length == strlen("crossed_boundary") &&
+		   strncmp(value, "crossed_boundary", length) == 0) {
 	    if (c_data->crossed_boundary)
 		return "true";
 	    else
 		return "false";
-	} else
-	if (length == strlen("doit") &&
-		strncmp(value, "doit", length) == 0) {
-		return TM_TEXT_DOIT;
-	} else
-	if (length == strlen("widget") &&
-		strncmp(value, "widget", length) == 0) {
+	} else if (length == strlen("doit") &&
+		   strncmp(value, "doit", length) == 0) {
+	    return TM_TEXT_DOIT;
+	} else if (length == strlen("widget") &&
+		   strncmp(value, "widget", length) == 0) {
     	    Tm_Widget *wPtr;
 
 	    XtVaGetValues(c_data->widget, XmNuserData, &wPtr, NULL);
 	    return wPtr->pathName;
-	} else
-	if (length == strlen("value") &&
-		strncmp(value, "value", length) == 0) {
+	} else if (length == strlen("value") &&
+		   strncmp(value, "value", length) == 0) {
 	    return Tm_XmStringToString(c_data->value);
-        } else
-
-        if (length == strlen("call_data") &&
-                strncmp(value, "call_data", length) == 0) {
+        } else if (length == strlen("call_data") &&
+		   strncmp(value, "call_data", length) == 0) {
     	    Tm_Widget *wPtr;
 
 	    sprintf(buf, "%d", c_data->position);
@@ -1229,45 +971,35 @@ getValue(pathName, w, event, call_data, value, length)
             return "ERROR in substitution";
 
             return Tm_Merge(n_all, all);
-
 	} else {
 	    return "ERROR in substitution";
 	}
-    } else
-
-    if (XtIsSubclass(w, xmCSTextWidgetClass)) {
+    } else if (XtIsSubclass(w, xmCSTextWidgetClass)) {
 	XmCSTextVerifyCallbackStruct *c_data =
-			(XmCSTextVerifyCallbackStruct *) call_data;
-
+	    (XmCSTextVerifyCallbackStruct *) call_data;
+	
 	if (length == strlen("doit") &&
-		strncmp(value, "doit", length) == 0) {
+	    strncmp(value, "doit", length) == 0) {
 		return TM_TEXT_DOIT;
-	} else
-	if (length == strlen("currInsert") &&
-		strncmp(value, "currInsert", length) == 0) {
+	} else if (length == strlen("currInsert") &&
+		   strncmp(value, "currInsert", length) == 0) {
 	    sprintf(buf, "%d", c_data->currInsert);
 	    return buf;
-	} else
-	if (length == strlen("newInsert") &&
-		strncmp(value, "newInsert", length) == 0) {
+	} else if (length == strlen("newInsert") &&
+		   strncmp(value, "newInsert", length) == 0) {
 	    sprintf(buf, "%d", c_data->newInsert);
 	    return buf;
-	} else
-	if (length == strlen("startPos") &&
-		strncmp(value, "startPos", length) == 0) {
-		return TM_TEXT_STARTPOS;
-	} else
-	if (length == strlen("endPos") &&
-		strncmp(value, "endPos", length) == 0) {
-		return TM_TEXT_ENDPOS;
-	} else
-	if (length == strlen("text") &&
-		strncmp(value, "text", length) == 0) {
-		return TM_TEXT_PTR;
-        } else
-
-        if (length == strlen("call_data") &&
-                strncmp(value, "call_data", length) == 0) {
+	} else if (length == strlen("startPos") &&
+		   strncmp(value, "startPos", length) == 0) {
+	    return TM_TEXT_STARTPOS;
+	} else if (length == strlen("endPos") &&
+		   strncmp(value, "endPos", length) == 0) {
+	    return TM_TEXT_ENDPOS;
+	} else if (length == strlen("text") &&
+		   strncmp(value, "text", length) == 0) {
+	    return TM_TEXT_PTR;
+        } else if (length == strlen("call_data") &&
+		   strncmp(value, "call_data", length) == 0) {
 
 	    all[n_all++] = make2list("doit", TM_TEXT_DOIT);
 
@@ -1276,17 +1008,11 @@ getValue(pathName, w, event, call_data, value, length)
 
 	    sprintf(buf, "%d", c_data->newInsert);
 	    all[n_all++] = make2list("newInsert", buf);
-
 	    all[n_all++] = make2list("startPos", TM_TEXT_STARTPOS);
-
 	    all[n_all++] = make2list("endPos", TM_TEXT_ENDPOS);
-
 	    all[n_all++] = make2list("ptr", TM_TEXT_PTR);
-
 	    all[n_all++] = make2list("length", TM_TEXT_LENGTH);
-
             return Tm_Merge(n_all, all);
-
 	} else {
 	    return "ERROR in substitution";
 	}
@@ -1299,7 +1025,8 @@ getValue(pathName, w, event, call_data, value, length)
      * does it belong in what someone else added?
      */
     return Tm_ExternExpandPercent(pathName, w, event,
-                		call_data, value, length);
+				  call_data, value, length);
+    return "";				    /* avoid comppiler warning */
 }
 
 
@@ -1401,11 +1128,14 @@ Tm_ExpandPercents(pathName, w, event, call_data, before)
 	} else {
 	    before++;
 	    wordSize = 0;
-	    while (isalpha(before[wordSize]) || before[wordSize] == '_') {
-		wordSize++;
+	    if (*before != '%') {
+	        while (isalpha(before[wordSize]) || before[wordSize] == '_') {
+	 	    wordSize++;
+	        }
 	    }
 	    if (wordSize == 0) {
 	        after = addChars(after, &afterSize, &end, &spaceLeft, 1, "%");
+		before++;
 	    } else {
 		value = getValue(pathName, w,
 				event, call_data, before, wordSize);
