@@ -19,6 +19,8 @@
 #include <Xm/Xm.h>
 #include <Xm/List.h>
 #include <Xm/Text.h>
+#include <Xm/DialogS.h>
+#include <X11/Shell.h>
 
 /* this is a hack because some converters need info about what
    types they are converting, and Xt does not give such info.
@@ -27,6 +29,20 @@
  */
 char *Tm_Resource = NULL;
 
+/*
+   While Motif copies all potentially "volatile" values such as
+   labelString and Text value, the Intrinsics do not - they assume
+   the data will stay around unmodified. This really only affects
+   Xt resources of type String, for which we have to make our own
+   copy, because tcl will destroy the one the Intrinsics rely on
+ */
+static char *must_copy_resource[] = {
+	XmNiconName,
+	XmNtitle,
+	XmNgeometry
+};
+  
+
 
 /*
  *--------------------------------------------------------------
@@ -226,8 +242,17 @@ Tm_ConvertValue(w, fromType, fromValue, fromSize,
     unsigned int toSize;
 {
     XrmValue from, to;
+    int n;
 
     if (strcmp(fromType, toType) == 0) {
+	/* do we have to copy the value so Xt dosen't stuff up? */
+	for (n = 0; n < XtNumber(must_copy_resource); n++) {
+	    if (strcmp(Tm_Resource, must_copy_resource[n]) == 0) {
+		*toValue = (XtArgVal) XtNewString(fromValue);
+		return True;
+	    }
+	}
+
 	*toValue = (XtArgVal) fromValue;
 	return TRUE;
     }
@@ -280,8 +305,6 @@ Tm_ConvertValueFromString(w, resources, num_resources,
     XtArgVal *new_value;
 {
     int n;
-    XrmValue from, converted;
-    XtArgVal tmpval;
     Boolean result;
 
 #   ifdef DEBUG
@@ -338,8 +361,7 @@ Tm_ConvertValueFromStringQuark(w, resources, num_resources,
 {
     int n;
     XrmValue from, converted;
-    XtArgVal tmpval;
-    int resourceQ;
+    XrmQuark resourceQ;
 
     resourceQ = XrmStringToQuark(resource);
 #   ifdef DEBUG
@@ -347,7 +369,7 @@ Tm_ConvertValueFromStringQuark(w, resources, num_resources,
 			orig_value, resource);
 #   endif
     for (n = 0; n < num_resources; n++) {
-	if (resourceQ == (int) resources->resource_name) {
+	if (resourceQ == (XrmQuark) resources->resource_name) {
 	    break;
 	}
 	resources++;
@@ -358,7 +380,7 @@ Tm_ConvertValueFromStringQuark(w, resources, num_resources,
 
     /* we have a match - convert and install the resource */
     /* special case - same type */
-    if (strcmp(XrmQuarkToString((int) resources->resource_type), XtRString) == 0) {
+    if (strcmp(XrmQuarkToString((XrmQuark) resources->resource_type), XtRString) == 0) {
 	*new_value =  (XtArgVal) orig_value;
 	return TRUE;
     }
@@ -368,7 +390,7 @@ Tm_ConvertValueFromStringQuark(w, resources, num_resources,
     converted.addr = NULL;
 
     if (!XtConvertAndStore(w, XtRString, &from,
-		XrmQuarkToString((int) resources->resource_type), &converted)) {
+		XrmQuarkToString((XrmQuark) resources->resource_type), &converted)) {
 	return FALSE;
     }
     *new_value = 0;
@@ -415,12 +437,12 @@ Tm_MaybeSetStringTableSize(w, resource)
     Widget w;
     String resource;
 {
+#if 0
     extern int Tm_XmStringTableSize;
     WidgetClass class;
     Arg arg;
     char *size_resource;
 
-#if 0
     class = XtClass(w);
     if (class == xmListWidgetClass) {
 	if (strcmp (resource, XmNitems) == 0) {
@@ -492,8 +514,8 @@ Tm_ConvertResourceToString(w, resource, resource_size, resource_type, value)
 	XtFree(from.addr);
 	return FALSE;
     }
-    bzero(value, sizeof(XtArgVal));
-    bcopy(converted.addr, value, converted.size);
+    bzero((char *) value, sizeof(XtArgVal));
+    bcopy(converted.addr, (char *) value, converted.size);
     
     XtFree(from.addr);
     return TRUE;
@@ -567,7 +589,7 @@ Tm_ConvertSubValueFromString(w, sec_resources, num_sec_resources,
     char *orig_value;
     XtArgVal *new_value;
 {
-    int n, length;
+    int n;
 
     for (n = 0; n < num_sec_resources; n++) {
        	if (Tm_ConvertValueFromString(w, sec_resources[n]->resources, 
@@ -602,7 +624,7 @@ Tm_ConvertSubValueToString(w, sec_resources, num_sec_resources,
     char *resource;
     char **new_value;
 {
-    int n, length;
+    int n;
 
     for (n = 0; n < num_sec_resources; n++) {
        	if (Tm_ConvertValueToString(w, sec_resources[n]->resources, 
@@ -680,11 +702,9 @@ Tm_SetValues(pathName, interp, w, parent, class, argv, argc, args, num_args)
     Cardinal num_resources;
     Cardinal num_constraint_resources;
     int num_values = 0;
-    XrmValue from, converted;
     XtArgVal new_value;
     char *resource;
     char *value;
-    Tm_ClientData *client_data;
 
     *num_args = 0;
 
@@ -694,6 +714,14 @@ Tm_SetValues(pathName, interp, w, parent, class, argv, argc, args, num_args)
 
     XtGetResourceList(class, &resources, &num_resources);
 #ifndef MOTIF11
+
+    /* hack in here, because of Motif bug in XmDialogShell getSecRes
+       which is set to NULL instead of inherit, so we can't get its
+       secondary resource data - they are merged with vendor shell stuff.
+     */
+    if (class == xmDialogShellWidgetClass)
+	class = vendorShellWidgetClass;
+
     num_sec_resources = XmGetSecondaryResourceData(class, &sec_resources);
 #endif
     if (parent != NULL && XtIsConstraint(parent)) {
@@ -739,11 +767,24 @@ Use \"%s\" method\n", resource);
 	    num_values++;
 	} else
 #endif
+	/* 
+	 * if the resource isn't found, we used to return an error like this
 	{
-	    sprintf(interp->result, "Conversion from String to \"%50s\" failed\n",
+	    sprintf(interp->result, "Conversion from String to \"%.50s\" failed\n",
                                 resource);
 	    return TCL_ERROR;
 	}
+	 *
+	 * this was prior to version 1.4. However, it breaks on certain Motif
+	 * 2.0 widgets, and also on some 1.2 functions like 
+	 * XmCreateFormDialog, where the same  resource list is handed to both
+	 * parent and child, relying upon Xt to ignore unknown resources. So
+	 * we better do what Xt recommends and ignore resources we can't find
+	 */
+	{
+		/* ignore resources we can't find */
+	}
+
 	argc -= 2;
 	argv += 2;
     }
@@ -792,12 +833,9 @@ Tm_GetValues(pathName, interp, w, class, argv, argc)
 #endif
     Cardinal num_resources;
     Cardinal num_constraint_resources;
-    int num_values = 0;
-    XrmValue from, converted;
     char *new_value;
 #   define MAXARGS 100
     char *resource;
-    Tm_ClientData *client_data;
     Widget parent;
 
     if (argc > MAXARGS*2) {
@@ -820,7 +858,7 @@ Tm_GetValues(pathName, interp, w, class, argv, argc)
 
     while (argc >= 2) {
 	if (argv[0][0] != '-') {
-	    sprintf(interp->result, "illegal setValues option \"%50s\"", argv[0]);
+	    sprintf(interp->result, "illegal getValues option \"%.50s\"", argv[0]);
 	    return TCL_ERROR;
 	}
 	resource = argv[0]+1;
@@ -831,8 +869,9 @@ Tm_GetValues(pathName, interp, w, class, argv, argc)
 #	    ifdef DEBUG
 	    fprintf(stderr, "Got value: %s\n", (char *) new_value);
 #	    endif
-	    if (Tcl_SetVar(interp, argv[1], (char *) new_value,
-			TCL_LEAVE_ERR_MSG) == NULL)
+	    if (Tcl_SetVar(interp, argv[1], 
+			   (new_value == NULL ? "" : new_value),
+			   TCL_LEAVE_ERR_MSG) == NULL)
 		fprintf(stderr, "%s\n", interp->result);
 	} else
         if (num_constraint_resources > 0 &&
@@ -840,7 +879,8 @@ Tm_GetValues(pathName, interp, w, class, argv, argc)
 	            num_constraint_resources, resource,
         	    &new_value)) {
 	    /* store in a variable now */
-	    if (Tcl_SetVar(interp, argv[1], (char *) new_value,
+	    if (Tcl_SetVar(interp, argv[1], 
+			(new_value == NULL ? "" : new_value),
 			TCL_LEAVE_ERR_MSG) == NULL)
 		fprintf(stderr, "%s\n", interp->result);
 	} else 
@@ -850,8 +890,9 @@ Tm_GetValues(pathName, interp, w, class, argv, argc)
 	            num_sec_resources, resource,
         	    &new_value)) {
 	    /* store in a variable now */
-	    if (Tcl_SetVar(interp, argv[1], (char *) new_value,
-			TCL_LEAVE_ERR_MSG) == NULL)
+	    if (Tcl_SetVar(interp, argv[1],
+			   (new_value == NULL ? "" : new_value),
+			   TCL_LEAVE_ERR_MSG) == NULL)
 		fprintf(stderr, "%s\n", interp->result);
 	} else
 #endif
@@ -896,6 +937,7 @@ Tm_GetResourceInfo(resourceInfo, w, resource)
     XtResource *resource;
 {
     char *resource_value;
+    int r;
 
     Tcl_DStringAppend(resourceInfo, "-", 1);
     Tcl_DStringAppend(resourceInfo, resource->resource_name, -1);
@@ -906,13 +948,14 @@ Tm_GetResourceInfo(resourceInfo, w, resource)
     /* hack to let resource converter know what this is */
     Tm_Resource = resource->resource_name;
 
-    if (Tm_ConvertResourceToString(w, resource->resource_name,
+    r = Tm_ConvertResourceToString(w, resource->resource_name,
 		resource->resource_size, resource->resource_type,
-		&resource_value)) {;
-        Tcl_DStringAppendElement(resourceInfo, resource_value);
-    } else {
-        Tcl_DStringAppendElement(resourceInfo, "");
-    }
+		&resource_value);
+
+    if (!r)
+	resource_value = "";
+
+    Tcl_DStringAppendElement(resourceInfo, resource_value);
 }
 /*
  *--------------------------------------------------------------
@@ -969,10 +1012,6 @@ Tm_ResourceList(interp, w, class)
 #endif
     Cardinal num_resources;
     Cardinal num_constraint_resources;
-    int num_values = 0;
-    char *new_value;
-    char *resource;
-    Tm_ClientData *client_data;
     Widget parent;
     int m, n;
     Tcl_DString resourceList, resourceInfo;
@@ -1092,7 +1131,7 @@ Tm_GetAppResources(interp, w, resource_str)
     for (n = 0; n < num_resources; n++) {
 	Tcl_SplitList(interp, resource_list[n], &num_elmts, &res_elmts);
 	if (num_elmts != 4) {
-	    sprintf(interp->result, "wrong getAppResources \"%50s\"",
+	    sprintf(interp->result, "wrong getAppResources \"%.50s\"",
 			resource_list[n]);
 	   return TCL_ERROR;
 	}
